<TeXmacs|1.0.3.9>

<style|mmxdoc>

<\body>
  <tmdoc-title|External libraries>

  One interesting feature of <value|mmx>, is its capability to plugin
  external code, coming for instance from specialized libraries. It allows
  you to combine the facility and power of a high level interpreter with the
  efficiency and functionnalties provided by external and dedicated tools.
  Let us describe how this <em|play-plug-and-play> \ approach works.

  <\enumerate-numeric>
    <item><em|play>: first, you write the code you want to plug in, for the
    moment only in <verbatim|C++>.

    <item><em|plug>: then, you define the correspondance between the types
    and functions in <verbatim|C++> and in <value|mmx>.

    <item><em|play>: finally, load the plugin module and play with new types
    and functions, that you bring into the interpreter.
  </enumerate-numeric>

  Let us detail steps 2 and 3.

  <section|Defining the correspondance between the external world and
  <value|mmx>>

  This correspondance is defined in a file <verbatim|mymodule.mmh> that will
  define a plugin module <verbatim|mymodule>. The content of such a file is
  described here.

  The language from which you import the functions and types is specified
  with the keyword <verbatim|foreign> (here only <verbatim|"C++"> is
  available):

  <\cpp-fragment>
    foreign "C++" {

    \ \ \ ...

    }
  </cpp-fragment>

  Preliminary inclusions or definitions, \ necessary for the compilation of
  the code can be specified with the keyword <verbatim|preamble>:

  <\cpp-fragment>
    foreign "C++" {

    \ \ \ preamble /"\ 

    \ \ \ typedef double NT;

    \ \ \ #include \<less\>synaps/latex.H\<gtr\>

    \ \ \ #include \<less\>synaps/upol.H\<gtr\>

    \ \ \ #include \<less\>gmpxx.h\<gtr\>

    \ \ \ void pretty_print(const UPolDse\<less\>NT\<gtr\> & p) {

    \ \ \ \ \ UPOL::print(ctex, p); }

    \ \ \ "/;

    \ \ \ ...

    }
  </cpp-fragment>

  Let us now detail how the correspondance \ between functions and types are
  defined (<verbatim|...> part above).

  <subsection|Defining new types>

  To define new types, use the keyword <verbatim|Type>:

  <\cpp-fragment>
    Double : Type == "double";

    UPol: Type == "UPolDse\<less\>double\<gtr\>";

    Integer : Type == "mpz_class";
  </cpp-fragment>

  The first line defines in <value|mmx> a type <verbatim|Double>, which
  corresponds to the <verbatim|C++> number type <verbatim|double>. The second
  line defines the <value|mmx> type <verbatim|UPol>, which corresponds to the
  type <verbatim|UPolDse\<less\>double\<gtr\>>, provided by the external
  library (here <samp|synaps>). The third line defines a <value|mmx> type
  <verbatim|Integer> corresponding to the type <verbatim|mpz_class> of large
  integers of the <samp|gmp> library.

  <subsection|Defining new functions>

  To defines new functions, you have to specify their signature in
  <value|mmx> and to give their external definition between \ <verbatim|" ...
  ";>

  <\cpp-fragment>
    infix + (v : Double, w: Double): Double == "v+w";

    UPol_(s: String) : UPol == "UPolDse\<less\>NT\<gtr\>(s)";

    gcd (a: Integer, b: Integer): Integer ==\ 

    \ \ \ "mpz_class r;

    \ \ \ \ mpz_gcd(r.get_mpz_t(),a.get_mpz_t(),b.get_mpz_t());

    \ \ \ \ return r";
  </cpp-fragment>

  Here is an example, where we defining an operator <verbatim|+> on the
  <verbatim|Double> type, a constructor of polynomials from
  <verbatim|String>, and a <verbatim|gcd> function on <verbatim|mpz_class>.\ 

  Notice that

  <\itemize-dot>
    <item>if there is only <em|one> instruction, you don't need to write a
    <verbatim|return> instruction,

    <item>if there is more than one instruction, you<em| have to> specify a
    <verbatim|return> object, except if the output type is <verbatim|Void>.
  </itemize-dot>

  In order to simplify their treatment, definition codes which are too long
  should be defined in the <verbatim|preample> part.\ 

  The possibility to import external modules already built with this
  mechanism is done by the keyword <verbatim|use>:

  <\mmx-fragment>
    use "Integer";
  </mmx-fragment>

  This load the definitions from the module <verbatim|Integer> into the
  module we are constructing.

  <section|Building the plugin module>

  Once these definitions are written in a file, say <verbatim|mymodule.mmh>,
  you have to compile it, using the command <verbatim|mmx-use>:

  <\quote-env>
    <framed-fragment|<verbatim|mmx-use my_module>>
  </quote-env>

  This will produce\ 

  <\itemize>
    <item>a file <verbatim|mmx-mymodule.cpp> containing the <verbatim|C++>
    code for the internal correspondances between types and functions.

    <item>a file <verbatim|mmx-mymodule.so> which is the dynamic library that
    will be loaded in <value|mmx>.
  </itemize>

  Once this compilation process is finished, you can use the module
  \ <verbatim|mymodule> in <value|mmx> with the command

  <\mmx-fragment>
    1\<gtr\> \ use "mymodule";

    Double : Class\ 

    \<less\>\<less\>*: (Ostream Double) -\<gtr\> Ostream

    ...
  </mmx-fragment>

  The file <verbatim|mmx-mymodule.so> will be searched in the directories
  specified by the environment variable <verbatim|LD_LIBRARY_PATH>.\ 

  Once this module is loaded, the functions and types that you have defined
  in <verbatim|mymodule.mmh> can \ be used in the interpreter as any other.

  <tmdoc-copyright|2003|Bernard Mourrain|Joris van der Hoeven>

  <tmdoc-license|Permission is granted to copy, distribute and/or modify this
  document under the terms of the <hlink|GNU General Public
  License|http://www.gnu.org/licenses/gpl.txt>. If you don't have this file,
  write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.>
</body>