<TeXmacs|1.0.2.9>

<style|mmxdoc>

<\body>
  <tmdoc-title|Classical strongly typed systems>

  Among others, we have studied the type systems of <name|Axiom/Aldor>,
  <name|C++>, <name|Scheme> and <name|Caml>. Let us briefly describe some of
  our feelings about the positive and negative sides of these type systems.

  <paragraph|Aldor>

  The <name|Aldor> language is based on categories or signatures. Genericity
  is achieved by explicitly declaring all methods which a generic type is
  allowed to use by means of a category. In order to use such a method it
  should be explicitly imported first. The advantages of this approach are

  <\itemize>
    <item>The user is forced to cleanly modelize the appropriate categories
    and the compiler checks whether the user conforms to the desired
    specifications.

    <item>The type checking during the compilation is extremely efficient,
    because only a scalar types and methods are visible, due to the import
    construct.
  </itemize>

  However, the first advantage also leads to several disadvantages concerning
  the flexibility of the type system:

  <\itemize>
    <item>A routine which has been written for a given category, cannot
    easily be reused for a subcategory, even if the code would allow to do
    this.

    <item>Minor variations of categories can lead to a wealth of categories
    which are difficult to master and organize.

    <item>The user has to explicitly design categories and keep track of
    importations all the time, which leads to considerable administrative
    burden.
  </itemize>

  As a consequence, <name|Aldor> is mainly useful if the theory that one
  wishes to program is well-modelized beforehand. However, the rigidity of
  the type system can become a problem for large scale projects and
  especially research projects where models may have to be adapted as a
  function of time. Other disadvantages of <name|Aldor> is that is hard to
  write an interpreter for it, that the type inheritance system is very rigid
  and that it lacks of support for implicit conversions. The latter
  draw-backs could easily be removed though.

  <paragraph|C++>

  A limited degree of genericity in <name|C++> is achieved through the use of
  templates. The advantages of templates are their flexibility: not much type
  checking is done over the quantified variabled and their use requires no
  particular effort from the user. However, all possible types which are used
  during the execution have to be known at compile time, so that the user
  cannot use any real generic objects. Furthermore, templates do not behave
  nicely in combination with other dirty C++ ``features'' like the
  <verbatim|#include> directive and the header-file mechanism.

  <paragraph|Scheme>

  The <name|Scheme> language is nice because if its extreme simplicity,
  flexibility and the power of its control structures based on continuations.
  As many computer algebra languages (which are often based on <name|Lisp>),
  all type checking is done dynamically, at run-time. This has the advantage
  that an interpreter is extremely easy to write, but that too few
  type-knowledge is given to the compiler in order to produce really
  efficient code. Another disadvantage of the impossibility to define typed
  variables is that is non-trivial to integrate overloading and implicit
  type-conversion into the language.

  <paragraph|Caml>

  The functional <name|Caml> language shares many of the advantages of the
  <name|Scheme> language, while providing a genuine type system. The
  <name|Caml> type system has the advantages that

  <\itemize>
    <item>The type-checking can be done at compile-time, which made it
    possible to write a highly efficient compiler.

    <item>The types of expressions can be determined automatically, so that
    the user does not need to care about the types in declarations.
  </itemize>

  In a sense, the second advantage is at the exact opposite of the rigid type
  system of <name|Aldor>: all necessary type information is determined from
  the code. However, this intelligence of <name|Caml> has the inconvenience
  that it may passes useful safety type checks. It also carries the
  disadvantage of <name|Scheme> that it is non-trivial to integrate
  overloading and implicit type-conversion.

  <tmdoc-copyright|2003|Joris van der Hoeven>

  <tmdoc-license|Permission is granted to copy, distribute and/or modify this
  document under the terms of the <hlink|GNU General Public
  License|http://www.gnu.org/licenses/gpl.txt>. If you don't have this file,
  write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  Boston, MA 02111-1307, USA.>
</body>

<\initial>
  <\collection>
    <associate|page-even|30mm>
    <associate|page-reduce-bot|15mm>
    <associate|page-reduce-right|25mm>
    <associate|page-reduce-left|25mm>
    <associate|page-top|30mm>
    <associate|page-right|30mm>
    <associate|par-width|150mm>
    <associate|page-odd|30mm>
    <associate|page-bot|30mm>
    <associate|page-reduce-top|15mm>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|toc-1|<tuple|<uninit>|?>>
    <associate|toc-2|<tuple|<uninit>|?>>
    <associate|toc-3|<tuple|<uninit>|?>>
    <associate|toc-4|<tuple|<uninit>|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <with|par-left|<quote|6fn>|font-size|<quote|0.84>|Aldor<value|toc-dots><pageref|toc-1>>

      <with|par-left|<quote|6fn>|font-size|<quote|0.84>|C++<value|toc-dots><pageref|toc-2>>
    </associate>
  </collection>
</auxiliary>