
/******************************************************************************
* MODULE     : glued.cc
* DESCRIPTION: Externally glued objects
* COPYRIGHT  : (C) 2004  Bernard Mourrain and Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MMX_GLUED_HH
#define __MMX_GLUED_HH
#include <symbol.hh>
#include <alias.hh>
namespace mmx {

typedef generic (*mmx_function) (generic);
void extern_declare (symbol var_name, generic val, generic type);
void extern_declare (symbol var_name, mmx_function fn, generic type);
void extern_type_converter (symbol name, generic type_id, mmx_function fn);
generic as_glue_type (generic type);
generic extern_eval (generic g);
// defined in evaluate.cc

template<typename T> struct glued;
template<typename T> ostream& operator << (ostream&, glued<T>);
template<typename T> glued<T> as_glued (generic g);

template<typename T>
struct glued_rep: generic_rep {
public:
  T data; // the glued data

protected:
  int Type ();
  void Print (ostream& out);
  // int Hash ();
  // bool Equal (generic g) const;
  // generic Copy ();

public:
  inline glued_rep (T r): data (r) {}

  friend struct glued<T>;
  friend ostream& operator << <> (ostream& out, glued<T> a);
  friend glued<T> as_glued<T> (generic g);
};

template<typename T>
struct glued {
protected:
  glued_rep<T>* rep;
  inline glued<T> (glued_rep<T>* r): rep (r) {}

public:
  inline glued<T> (T r): rep (new glued_rep<T> (r)) {}
  inline operator generic () { return generic (rep); }
  inline glued_rep<T>* operator -> () { return rep; }
  inline bool operator == (glued<T> g) { return (rep->data == g->data); }
  inline bool operator != (glued<T> g) { return (rep->data != g->data); }

  friend struct glued_rep<T>;
  friend ostream& operator << <> (ostream& out, glued<T> x);
  friend glued<T> as_glued<T> (generic g);
};

/******************************************************************************
* Required routines for derived classes from generic_rep
******************************************************************************/

template<typename T> int
glued_rep<T>::Type () {
  return GLUED_TYPE;
}

template<typename T> void
glued_rep<T>::Print (ostream& out) {
  out << glued<T> (this);
}

template<typename T> ostream&
operator << (ostream& out, glued<T> x) {
  return out << x->data;
}

/******************************************************************************
* Conversions
******************************************************************************/

template<typename T> inline glued<T>
as_glued (generic g) {
  if (gentype(g) != GLUED_TYPE) { // in order to support re-entrance
    // cout << "Warning: " << g << " considered as glue" << lf;
    glued<generic>* ptr= new glued<generic> (g);
    return *reinterpret_cast<glued<T>*> (ptr);
  }
  return glued<T> ((glued_rep<T>*) g.rep); 
}

template<typename T> T
view_as (generic g) {  
  return as_glued<T>(g) -> data;
}

template<typename T> T
view_as_inst (generic g) {  
  return T ((void*) &(as_glued<int>(g) -> data));
}

/******************************************************************************
* Glued aliases
******************************************************************************/

template<typename T> class glued_alias;
template<typename T> glued_alias<T> as_glued_alias (generic g);

template<typename T>
class glued_alias_rep: public genalias_rep {
protected:
  T& ref;
  inline int Type () { return GLUED_ALIAS_TYPE; }

public:
  inline glued_alias_rep (T& ref2): ref (ref2) {}
  inline generic get () { return glued<T> (ref); }
  inline void set (generic val) { ref= view_as<T> (val); }

  friend class glued_alias;
};

template<typename T>
class glued_alias {
protected:
  glued_alias_rep<T>* rep;
  inline glued_alias (glued_alias_rep<T>* rep2): rep (rep2) {}

public:
  inline glued_alias (T& ref);
  inline operator generic () { return generic (rep); }
  inline glued_alias_rep<T>* operator -> () { return rep; }

  friend class glued_alias_rep;
  friend glued_alias<T> as_glued_alias<T> (generic g);
};

template<typename T> inline
glued_alias<T>::glued_alias<T> (T& ref):
  rep (new glued_alias_rep<T> (ref)) {}

template<typename T> inline glued_alias<T>
as_glued_alias (generic g) {
  if (gentype(g) != GLUED_ALIAS_TYPE) type_mismatch (g, "glued_alias");
  return glued_alias<T> ((glued_alias_rep<T>*) g.rep); }

template<typename T> T&
view_as_alias (generic g) {
  generic ref= as_genalias (g) -> get ();
  T& ret= as_glued<T> (ref) -> data;
  return ret;
}

template<typename T> T&
view_as_alias_inst (generic g) {  
  generic ref= as_genalias (g) -> get ();
  int& ret= as_glued<int> (ref) -> data;
  T* rep= new T ((void*) &ret);
  return *rep;
}

/******************************************************************************
* Glued tuples and vtuples
******************************************************************************/

// ATTENTION:
// tuples  are internally stored as (tuple! item_1 ... item_n)
// vtuples are internally stored as (vtuple! tuple_1 ... tuple_r)

template<typename T>
class glued_tuple {
  generic rep;

public:
  inline glued_tuple (generic g): rep (g) {}
  inline operator generic () { return rep; }
  inline T operator [] (int i) { return view_as<T> (rep[i+1]); }
};

template<typename T> inline generic
as_generic (glued_tuple<T> g) {
  return (generic) g;
}

template<typename T> inline int
N (glued_tuple<T> g) {
  return N ((generic) g) - 1;
}

template<typename T>
class glued_tuple<glued_tuple<T> > {
  generic rep;

public:
  inline glued_tuple<glued_tuple<T> > (generic g): rep (g) {}
  inline operator generic () { return rep; }
  inline glued_tuple<T> operator [] (int i) {
    return glued_tuple<T> (rep[i+1]); }
};

template<typename T> inline generic
as_generic (glued_tuple<glued_tuple<T> > g) {
  return (generic) g;
}

template<typename T> inline int
N (glued_tuple<glued_tuple<T> > g) {
  return N ((generic) g) - 1;
}

} //namespace mmx
#endif // __MMX_GLUED_HH
