
/******************************************************************************
* MODULE     : gentable.hh
* DESCRIPTION: generic tables
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __GENTABLE_HH
#define __GENTABLE_HH
#include <list.hh>
namespace mmx {

struct gentable_rep: public generic_rep {
  inline gentable_rep () {}
  inline virtual ~gentable_rep () {}

  virtual int  Arity ();
  virtual bool Equal (generic g);
  virtual void Print (ostream& out);

  virtual generic  get_binding (generic key); // returns binding pair for key
  virtual void     set_binding (generic binding); // set binding pair for key
  virtual void     reset (generic key);       // destroy binding for key
  virtual list     entries ();                // all entries of table as list
  virtual bool     contains (generic key);
  virtual generic  read (generic key);
  virtual generic& write (generic key);
  virtual bool     local_contains (generic key); // don't overload
  virtual generic  local_read (generic key);     // global definitions
  virtual generic& local_write (generic key);    // in local envs
  virtual int      local_level ();               // nesting level

  friend class table;
  friend class loctable;
};

struct gentable {
  gentable_rep* rep;
  inline gentable (gentable_rep* rep2): rep (rep2) {}

  inline friend gentable as_gentable (generic g) {
    if ((gentype(g) < TABLE_TYPE) || (gentype(g) > XTABLE_TYPE))
      type_mismatch (g, "gentable");
    return gentable ((gentable_rep*) g.rep); }
  inline operator generic () { return generic (rep); }
  inline gentable_rep* operator -> () { return rep; }

  inline generic  operator [] (generic g) { return rep->read (g); }
  inline generic& operator () (generic g) { return rep->write (g); }
  friend bool     operator == (gentable a, gentable b);
  friend bool     operator != (gentable a, gentable b);
  friend ostream& operator << (ostream& out, gentable a);
  friend void     dump (gentable t);
};

}
#endif // __GENTABLE_HH
